/*
 * flashStorage.c
 *
 *  Created on: 14.01.2019
 *      Author: Procesorowiec
 */


#include "system8xx.h"
#include "flashStorage.h"


/* Last sector address */
//#define START_ADDR_LAST_SECTOR  0x1C00			//for 8k flash
//#define IAP_LAST_SECTOR         7

#define FLASHSTORAGE_START_ADDR_LAST_SECTOR  	0x7C00				//for 32k flash
#define FLASHSTORAGE_LAST_SECTOR   	31

#define FLASHSTORAGE_SECTOR_SIZE    1024

#define FLASHSTORAGE_BYTESIZE		256
#define FLASHSTORAGE_ELEMENT_SIZE	2
#define FLASHSTORAGE_ELEMENTS		128

#define FLASHSTORAGE_UPDATE_DELAY	400


uint32_t storageBuff[(FLASHSTORAGE_BYTESIZE / 4)];
uint32_t *storageBuffSrc;

int16_t storageNeedUpdate;



uint8_t StorageEraseSector(uint32_t strSector, uint32_t endSector);
uint8_t StorageCopyRamToFlash(uint32_t dstAdd, uint32_t *srcAdd, uint32_t byteswrt);
uint8_t StoragePreSectorForReadWrite(uint32_t strSector, uint32_t endSector);


//----------------------------------------------------
void StorageBuffSrcSet(uint32_t *buffSrc){
	if (buffSrc > 0){
		storageBuffSrc = buffSrc;
	} else {
		storageBuffSrc = storageBuff;
	}
}

//----------------------------------------------------
void StorageInit(void){

	StorageBuffSrcSet(0);

	int16_t *storageSrc = FLASHSTORAGE_START_ADDR_LAST_SECTOR;
	int16_t *storageDst = storageBuffSrc;

	for(uint8_t i = 0; i<FLASHSTORAGE_ELEMENTS; i++){
		*storageDst = *storageSrc;
		storageDst++;
		storageSrc++;
	}
}

//----------------------------------------------------
void StorageFill(void){
	int16_t *storageDst = storageBuffSrc;
	for(uint8_t i = 0; i<FLASHSTORAGE_ELEMENTS; i++){
		*storageDst = i;
		storageDst++;
	}
}

//----------------------------------------------------
void StorageValAdd(int16_t data, int16_t adr){
	int16_t *storageDst;

	if (adr < FLASHSTORAGE_ELEMENTS){
		storageDst = storageBuffSrc;
		storageDst += adr;
		*storageDst = data;
		storageNeedUpdate = FLASHSTORAGE_UPDATE_DELAY;
	}
}

//----------------------------------------------------
void StorageManyValAdd(int16_t *data, int16_t len, int16_t adr){
	int16_t *storageDst = storageBuffSrc;
	storageDst += adr;

	for (;len>0;len--){
		*storageDst = *data;
		storageDst++;
		data++;
	}

	storageNeedUpdate = FLASHSTORAGE_UPDATE_DELAY;
}

//----------------------------------------------------
int16_t StorageValGet(int16_t adr){
	int16_t *storageSrc;

	if (adr < FLASHSTORAGE_ELEMENTS){
		storageSrc = storageBuffSrc;
		storageSrc += adr;
		return *storageSrc;
	}
	return 0;
}

//----------------------------------------------------
void StorageManyValGet(int16_t *dst, int16_t len, int16_t adr){
	int16_t *storageSrc = storageBuffSrc;
	storageSrc += adr;

	for (;len>0;len--){
		 *dst= *storageSrc;
		storageSrc++;
		dst++;
	}
}

//----------------------------------------------------
void StorageNeedUpdateSet(void){
	storageNeedUpdate = FLASHSTORAGE_UPDATE_DELAY;
}

//----------------------------------------------------
int16_t StorageProc(void){
	if (storageNeedUpdate > 0){
		storageNeedUpdate--;
		if (storageNeedUpdate == 0){

			StorageSave();
			return 0;
		}
	}
	return 0;
}

//----------------------------------------------
void StorageSave(void){
	uint8_t result;

	__disable_irq();

	result = StoragePreSectorForReadWrite(FLASHSTORAGE_LAST_SECTOR, FLASHSTORAGE_LAST_SECTOR);
	SerialDebugAddParam("\rPRE1", result);

	if (result == FLASHSTORAGE_CMD_SUCCESS)
	{
		result = StorageEraseSector(FLASHSTORAGE_LAST_SECTOR, FLASHSTORAGE_LAST_SECTOR);
		SerialDebugAddParam(" ERS", result);

		result = StoragePreSectorForReadWrite(FLASHSTORAGE_LAST_SECTOR, FLASHSTORAGE_LAST_SECTOR);
		SerialDebugAddParam(" PRE2", result);

		if (result == FLASHSTORAGE_CMD_SUCCESS)
		{
			result = StorageCopyRamToFlash(FLASHSTORAGE_START_ADDR_LAST_SECTOR, storageBuffSrc, FLASHSTORAGE_BYTESIZE);
			SerialDebugAddParam(" COPY", result);

			//Chip_FMC_ComputeSignatureBlocks(FLASHSTORAGE_START_ADDR_LAST_SECTOR, (FLASHSTORAGE_SECTOR_SIZE / 16));
			//while (Chip_FMC_IsSignatureBusy()) {}
			//SerialText("\r5:"); SerialValue(Chip_FMC_GetSignature(0));
			//if (result == IAP_CMD_SUCCESS) SerialText("\rSave OK\r");
		}
	}
	__enable_irq();
}




/* Erase sector */
uint8_t StorageEraseSector(uint32_t strSector, uint32_t endSector){
	uint32_t command[5], result[5];

	command[0] = FLASHSTORAGE_ERSSECTOR_CMD;
	command[1] = strSector;
	command[2] = endSector;
	command[3] = MAIN_CLOCK / 1000;
	iap_entry(command, result);

	return result[0];
}

/* Copy RAM to flash */
uint8_t StorageCopyRamToFlash(uint32_t dstAdd, uint32_t *srcAdd, uint32_t byteswrt){
	uint32_t command[5], result[5];

	command[0] = FLASHSTORAGE_WRISECTOR_CMD;
	command[1] = dstAdd;
	command[2] = (uint32_t) srcAdd;
	command[3] = byteswrt;
	command[4] = MAIN_CLOCK / 1000;
	iap_entry(command, result);

	return result[0];
}

/* Prepare sector for write operation */
uint8_t StoragePreSectorForReadWrite(uint32_t strSector, uint32_t endSector){
	uint32_t command[5], result[5];

	command[0] = FLASHSTORAGE_PREWRRITE_CMD;
	command[1] = strSector;
	command[2] = endSector;
	iap_entry(command, result);

	return result[0];
}
