/*
 * leds.c
 *
 *  Created on: 25.08.2018
 *      Author: Procesorowiec
 */


#include "system8xx.h"
#include "leds.h"

#define LED1_PORT			0
#define LED1_PIN			12

#define NUM_OF_LEDS			1
#define LEDS_ON				0
#define LEDS_OFF			1


#define LEDS_PERIOD			100			//ms


PINEX_STRUCT_T ledsArray[NUM_OF_LEDS] = {
		{LED1_PORT, LED1_PIN, (IOCON_MODE_INACT | IOCON_OD_ENABLE), PIN_OUTPUT_EN, LEDS_ON, LEDS_OFF},
};

uint8_t ledsStateArray[NUM_OF_LEDS];

//--------------------------------------------------------------------------
void LedsInit(void){
	SystemPortInit(ledsArray, NUM_OF_LEDS);
}

//--------------------------------------------------------------------------
void LedsProc(void){
	static uint8_t ledsPeriod;

	if (ledsPeriod < (LEDS_PERIOD / MAIN_DELAY)){
		ledsPeriod++;
	} else {
		ledsPeriod = 0;

		for(uint8_t i=0; i<NUM_OF_LEDS; i++){
			if ((ledsStateArray[i] > 0) && (ledsStateArray[i] < 0xFF)){
				ledsStateArray[i]--;
				if (ledsStateArray[i] == 0){
					Chip_GPIO_SetPinState(LPC_GPIO_PORT, ledsArray[i].port, ledsArray[i].pin, ledsArray[i].idle);
				}
			}
		}
	}
}

//--------------------------------------------------------------------------
//ch 1...6
void LedsSet(uint8_t ch, uint8_t time){
	if ((ch > 0) && (ch <= NUM_OF_LEDS)){
		ch -= 1;
		ledsStateArray[ch] = time;
		if (time > 0){
			Chip_GPIO_SetPinState(LPC_GPIO_PORT, ledsArray[ch].port, ledsArray[ch].pin, ledsArray[ch].active);
		} else {
			Chip_GPIO_SetPinState(LPC_GPIO_PORT, ledsArray[ch].port, ledsArray[ch].pin, ledsArray[ch].idle);
		}
	}
}

