/*
 * mypwm.c
 *
 *  Created on: Mar 1, 2018
 *      Author: dsosnowski
 */

#include "chip.h"
#include "system8xx.h"
#include "mypwm.h"

#define MYPWM_TIMER			LPC_SCT
#define PWM_CLOCK			MAIN_CLOCK

#define PWM_RATE			10000
#define PWM_CHANNEL			1
#define PWM_OUTPUT			0


static inline void MyPwmSetMatchCount(uint8_t channel, uint32_t value);
static inline void MyPwmSetMatchReload(uint8_t channel, uint32_t value);
static uint32_t MyPwmPercent2Ticks(uint8_t val);
static uint32_t MyPwmByte2Ticks(uint8_t val);

//---------------------------------------------------------
void MyPwmInit(void){
	Chip_Clock_EnablePeriphClock(SYSCTL_CLOCK_SCT);
	Chip_SYSCTL_PeriphReset(RESET_SCT);
	MyPwmRateSet(PWM_RATE);
	Chip_SCTPWM_Start(MYPWM_TIMER);
}

//---------------------------------------------------------
void MyPwmRateSet(uint32_t freq){
	uint32_t rate;
	rate = (PWM_CLOCK / freq);

	/* Stop the SCT before configuration */
	Chip_SCTPWM_Stop(MYPWM_TIMER);

	/* Set MATCH0 for max limit */
	MYPWM_TIMER->REGMODE_U = 0;
	MyPwmSetMatchCount(0, 0);
	MyPwmSetMatchReload(0, rate);
	MYPWM_TIMER->EV[0].CTRL = 1 << 12;
	MYPWM_TIMER->EV[0].STATE = 1;

	/* Set SCT Counter to count 32-bits and reset to 0 after reaching MATCH0 */
	Chip_SCT_Config(MYPWM_TIMER, SCT_CONFIG_32BIT_COUNTER | SCT_CONFIG_AUTOLIMIT_U);
}

//---------------------------------------------------------
static inline void MyPwmSetMatchCount(uint8_t channel, uint32_t value){
	MYPWM_TIMER->MATCH[channel].U = value;
}
//---------------------------------------------------------
static inline void MyPwmSetMatchReload(uint8_t channel, uint32_t value){
	MYPWM_TIMER->MATCHREL[channel].U = value;
}

//---------------------------------------------------------
//channel 0...5
void MyPwmSetOutPin(uint8_t channel, uint8_t pin){
	uint32_t ch = (uint32_t)channel;
	uint32_t reg = (ch + 1);	//ch 0 reserved for counter limit

	MYPWM_TIMER->EV[reg].CTRL = (reg | (1 << 12));
	MYPWM_TIMER->EV[reg].STATE = 1;
	MYPWM_TIMER->OUT[ch].SET = 1;
	MYPWM_TIMER->OUT[ch].CLR = 1 << reg;

	/* Clear the output in-case of conflict */
	MYPWM_TIMER->RES = (MYPWM_TIMER->RES & ~(3 << (ch << 1))) | (0x01 << (ch << 1));

	/* Set and Clear do not depend on direction */
	MYPWM_TIMER->OUTPUTDIRCTRL = (MYPWM_TIMER->OUTPUTDIRCTRL & ~((3 << (ch << 1))|SCT_OUTPUTDIRCTRL_RESERVED));

	switch (ch){
		case 0: Chip_SWM_MovablePinAssign(SWM_SCT_OUT0_O, pin); break;
		case 1: Chip_SWM_MovablePinAssign(SWM_SCT_OUT1_O, pin); break;
		case 2: Chip_SWM_MovablePinAssign(SWM_SCT_OUT2_O, pin); break;
		case 3: Chip_SWM_MovablePinAssign(SWM_SCT_OUT3_O, pin); break;
		case 4: Chip_SWM_MovablePinAssign(SWM_SCT_OUT4_O, pin); break;
		case 5: Chip_SWM_MovablePinAssign(SWM_SCT_OUT5_O, pin); break;
	}
}

//---------------------------------------------------------
void MyPwmDutyPercentSet(uint8_t channel, uint8_t val){
	MyPwmSetMatchReload((channel + 1), MyPwmPercent2Ticks(val));
}
//---------------------------------------------------------
void MyPwmDutyByteSet(uint8_t channel, uint8_t val){
	MyPwmSetMatchReload((channel + 1), MyPwmBytet2Ticks(val));
}
//---------------------------------------------------------
static uint32_t MyPwmPercent2Ticks(uint8_t val){
	return (((MYPWM_TIMER->MATCHREL[0].U) * val) / 100);
}
//---------------------------------------------------------
static uint32_t MyPwmByte2Ticks(uint8_t val){
	return (((MYPWM_TIMER->MATCHREL[0].U) * val) / 255);
}
