/*
 * serial8xx.c
 *
 *  Created on: 26.08.2018
 *      Author: Procesorowiec
 */


#include "system8xx.h"
#include "serial8xx.h"



#define SERIAL_CLOCK		PREMAIN_CLOCK
#define SERIAL_PRESC		2


//----------------------------------------------------------------------------------
/* Initialize the UART peripheral */
void SerialInit(LPC_USART_T *pUART){
	Chip_Clock_SetUARTClockDiv(SERIAL_PRESC);

	/* UART reset */
	if (pUART == LPC_USART0) {
		/* Enable USART clock */
		Chip_Clock_EnablePeriphClock(SYSCTL_CLOCK_UART0);
		/* Peripheral reset control to USART0 */
		Chip_SYSCTL_PeriphReset(RESET_USART0);
	} else if (pUART == LPC_USART1) {
		/* Enable USART clock */
		Chip_Clock_EnablePeriphClock(SYSCTL_CLOCK_UART1);
		/* Peripheral reset control to USART1 */
		Chip_SYSCTL_PeriphReset(RESET_USART1);
	} else {
		/* Enable USART clock */
		Chip_Clock_EnablePeriphClock(SYSCTL_CLOCK_UART2);
		/* Peripheral reset control to USART2 */
		Chip_SYSCTL_PeriphReset(RESET_USART2);
	}
}

//----------------------------------------------------------------------------------
/* @brief	Configure data width 8, parity none and stop bits 1*/
inline void SerialDataConfig(LPC_USART_T *pUART){
	pUART->CFG = (UART_CFG_ENABLE | UART_CFG_DATALEN_8 | UART_CFG_PARITY_NONE | UART_CFG_STOPLEN_1);
}

//----------------------------------------------------------------------------------
/* Set baud rate for UART */
inline void SerialBaudSet(LPC_USART_T *pUART, uint32_t baudrate){
	pUART->BRG = (((SERIAL_CLOCK / SERIAL_PRESC) / (16 * baudrate)) - 1);
}

//----------------------------------------------------------------------------------
inline void SerialEnable(LPC_USART_T *pUART){
	pUART->CFG |= UART_CFG_ENABLE;
}

//----------------------------------------------------------------------------------
void SerialTenPinEnable(LPC_USART_T *pUART){
	pUART->CFG |= (1<<20 | 1<<21);
	//OETA, OESEL, OEPOL
}

//----------------------------------------------------------------------------------
inline void SerialTXEnable(LPC_USART_T *pUART){
	pUART->CTRL &= ~UART_CTRL_TXDIS;
}
//----------------------------------------------------------------------------------
inline void SerialSend(LPC_USART_T *pUART, uint8_t data){
	pUART->TXDATA = (uint32_t)data;
}
//----------------------------------------------------------------------------------
inline uint8_t SerialReadGet(LPC_USART_T *pUART){
	return (uint8_t)pUART->RXDATA;
}
//----------------------------------------------------------------------------------
inline uint32_t SerialStatusGet(LPC_USART_T *pUART){
	return (pUART->STAT);
}
//----------------------------------------------------------------------------------
uint8_t SerialReadReadyCheck(LPC_USART_T *pUART){
	if ((pUART->STAT & UART_STAT_RXRDY) > 0) return 1;
	return 0;
}
//----------------------------------------------------------------------------------
uint8_t SerialReadIdleCheck(LPC_USART_T *pUART){
	if ((pUART->STAT & UART_STAT_RXIDLE) > 0) return 1;
	return 0;
}
//----------------------------------------------------------------------------------
uint8_t SerialSendReadyCheck(LPC_USART_T *pUART){
	if ((pUART->STAT & UART_STAT_TXRDY) > 0) return 1;
	return 0;
}
//----------------------------------------------------------------------------------
uint8_t SerialSendIdleCheck(LPC_USART_T *pUART){
	if ((pUART->STAT & UART_STAT_TXIDLE) > 0) return 1;
	return 0;
}

//----------------------------------------------------------------------------------
inline void SerialReadIntEnable(LPC_USART_T *pUART){
	pUART->INTENSET |= (UART_INTEN_RXRDY);
}
//----------------------------------------------------------------------------------
inline void SerialReadIntDisable(LPC_USART_T *pUART){
	pUART->INTENCLR |= (UART_INTEN_RXRDY);
}
//----------------------------------------------------------------------------------

inline void SerialSendIntEnable(LPC_USART_T *pUART){
	pUART->INTENSET |= (UART_INTEN_TXRDY);
}
//----------------------------------------------------------------------------------
inline void SerialSendIntDisable(LPC_USART_T *pUART){
	pUART->INTENCLR |= (UART_INTEN_TXRDY);
}

//----------------------------------------------------------------------------------
inline void SerialSendIdleIntEnable(LPC_USART_T *pUART){
	pUART->INTENSET |= (UART_INTEN_TXIDLE);
}
//----------------------------------------------------------------------------------
inline void SerialSendIdleIntDisable(LPC_USART_T *pUART){
	pUART->INTENCLR |= (UART_INTEN_TXIDLE);
}

//----------------------------------------------------------------------------------
inline uint32_t SerialIntStatusGet(LPC_USART_T *pUART){
	return (pUART->INTSTAT);
}


//----------------------------------------------------------------------------------
/* UART receive/transmit interrupt handler for ring buffers */
//void Chip_UART_IRQRBHandler(LPC_USART_T *pUART, RINGBUFF_T *pRXRB, RINGBUFF_T *pTXRB)
void SerialIrqHandler(LPC_USART_T *pUART, STRINGBUFF_T *txStringBuff,  STRINGBUFF_T *rxStringBuff){
	uint8_t data;
	uint32_t status = SerialStatusGet(pUART);

	/* Handle transmit interrupt if enabled */
	//if (SerialSendReadyCheck(pUART) > 0) {
	if ((status & UART_STAT_TXRDY) > 0){
		if (txStringBuff->track < txStringBuff->i){
			data = *(txStringBuff->base + txStringBuff->track);
			txStringBuff->track++;
			SerialSend(pUART, data);
		}
		if (txStringBuff->track >= txStringBuff->i){
			SerialSendIntDisable(pUART);
			//txStringBuff.track = 0;
		}
	}

	/* Handle receive interrupt */
	//if (SerialReadReadyCheck(pUART) > 0){
	if ((status & UART_STAT_RXRDY) > 0){
		data = SerialReadGet(pUART);
		StringBuilderAddChar(rxStringBuff, data);
	}
}

//----------------------------------------------------------------------------------
/* UART receive/transmit interrupt handler for ring buffers */
//void Chip_UART_IRQRBHandler(LPC_USART_T *pUART, RINGBUFF_T *pRXRB, RINGBUFF_T *pTXRB)
void SerialIrqHandlerBlockMode(LPC_USART_T *pUART, STRINGBUFF_T *txStringBuff,  STRINGBUFF_T *rxStringBuff){
	uint8_t data;
	uint32_t status = SerialIntStatusGet(pUART);

	/* Handle transmit interrupt if enabled */
	//if (SerialSendReadyCheck(pUART) > 0) {
	if ((status & UART_STAT_TXRDY) > 0){
		if (txStringBuff->track < txStringBuff->i){
			SerialReadIntDisable(pUART);
			data = *(txStringBuff->base + txStringBuff->track);
			txStringBuff->track++;
			SerialSend(pUART, data);
		}
		if (txStringBuff->track >= txStringBuff->i){
			SerialSendIntDisable(pUART);
			SerialSendIdleIntEnable(pUART);
			//txStringBuff.track = 0;
		}
	}

	/* Handle receive interrupt */
	//if (SerialReadReadyCheck(pUART) > 0){
	if ((status & UART_STAT_RXRDY) > 0){
		data = SerialReadGet(pUART);
		StringBuilderAddChar(rxStringBuff, data);
	}

	if ((status & UART_STAT_TXIDLE) > 0){
		SerialSendIdleIntDisable(pUART);
		data = SerialReadGet(pUART);
		SerialReadIntEnable(pUART);
	}
}

//----------------------------------------------------------------------------------
/* UART receive/transmit interrupt handler for ring buffers */
//void Chip_UART_IRQRBHandler(LPC_USART_T *pUART, RINGBUFF_T *pRXRB, RINGBUFF_T *pTXRB)
void SerialIrqHandlerCommandMode(LPC_USART_T *pUART, STRINGBUFF_T *txStringBuff,  COMMAND_REC_BUFF_T *rxCommandBuff){
	uint8_t data;
	uint32_t status = SerialIntStatusGet(pUART);

	/* Handle transmit interrupt if enabled */
	//if (SerialSendReadyCheck(pUART) > 0) {
	if ((status & UART_STAT_TXRDY) > 0){
		if (txStringBuff->track < txStringBuff->i){
			SerialReadIntDisable(pUART);
			data = *(txStringBuff->base + txStringBuff->track);
			txStringBuff->track++;
			SerialSend(pUART, data);
		}
		if (txStringBuff->track >= txStringBuff->i){
			SerialSendIntDisable(pUART);
			SerialSendIdleIntEnable(pUART);
			//txStringBuff.track = 0;
		}
	}

	/* Handle receive interrupt */
	//if (SerialReadReadyCheck(pUART) > 0){
	if ((status & UART_STAT_RXRDY) > 0){
 		data = SerialReadGet(pUART);
		CommandManAdd(rxCommandBuff, data);
	}

	if ((status & UART_STAT_TXIDLE) > 0){
		SerialSendIdleIntDisable(pUART);
		data = SerialReadGet(pUART);
		SerialReadIntEnable(pUART);
	}
}

//----------------------------------------------------------------------------------
void SerialSendHolding(LPC_USART_T *pUART, uint8_t *src){
	while (*src > 0){
		while (SerialSendReadyCheck(pUART) == 0){}
		SerialSend(pUART, *src);
		src++;
	}
}

//----------------------------------------------------------------------------------
void SerialSendStringBuffHolding(LPC_USART_T *pUART, STRINGBUFF_T *txStringBuff){
	uint8_t data;

	while (txStringBuff->track < txStringBuff->i){
		while (SerialSendReadyCheck(pUART) == 0){}
		data = *(txStringBuff->base + txStringBuff->track);
		txStringBuff->track++;
		SerialSend(pUART, data);
	}
}
