/*
 * simpleDebug.c
 *
 *  Created on: 26.08.2018
 *      Author: Procesorowiec
 */

#include "system8xx.h"
#include "serialDebug.h"
#include "commandMan.h"


#define DEBUG_USART       	LPC_USART0
#define DEBUG_IRQ	      	UART0_IRQn
#define DEBUG_HANDLER  		UART0_IRQHandler
#define DEBUG_BAUDRATE 		115200

#define DEBUG_BUFF_LEN		512
//#define DEBUG_COMMAND_PREFIX	COMMAND_PREFIX
//#define DEBUG_COMMAND_POSTFIX	COMMAND_POSTFIX

uint8_t debugBuff[DEBUG_BUFF_LEN];
STRINGBUFF_T debugString;

uint8_t debugCommandBuff[DEBUG_BUFF_LEN];
COMMAND_REC_BUFF_T debugCommandString;

COMMAND_STRUCT_T debugRecCommand;


#define SERIAL_PORT			0
#define SERIAL_TXD_PIN		4
#define SERIAL_RXD_PIN		0

PINEX_STRUCT_T serialPinsArray[2] = {
		{SERIAL_PORT, SERIAL_TXD_PIN, (IOCON_MODE_INACT), PIN_OUTPUT_EN, 0, 1},
		{SERIAL_PORT, SERIAL_RXD_PIN, (IOCON_MODE_PULLUP), PIN_INPUT, 0, 1}
};


//------------------------------------------------------------------
void SerialDebugInit(void){
	SystemPortInit(serialPinsArray, 2);

	/* UART signal muxing via SWM */
	Chip_SWM_MovablePinAssign(SWM_U0_TXD_O, 4);
	Chip_SWM_MovablePinAssign(SWM_U0_RXD_I, 0);

	StringBuilderStringInit(&debugString, debugBuff, DEBUG_BUFF_LEN);
	CommandManInit(&debugCommandString, debugCommandBuff, DEBUG_BUFF_LEN,
					COMMAND_PREFIX, COMMAND_POSTFIX);

	SerialInit(DEBUG_USART);
	SerialDataConfig(DEBUG_USART);
	SerialBaudSet(DEBUG_USART, DEBUG_BAUDRATE);
	SerialEnable(DEBUG_USART);
	SerialTXEnable(DEBUG_USART);
	SerialReadIntEnable(DEBUG_USART);
	NVIC_EnableIRQ(DEBUG_IRQ);
}

void DEBUG_HANDLER(void){
	SerialIrqHandlerCommandMode(DEBUG_USART, &debugString, &debugCommandString);
}

//------------------------------------------------------------------
uint8_t *SerialDebugGet(void){
	return StringBuilderGet(&debugString);
}

//------------------------------------------------------------------
void SerialDebugClear(void){
	StringBuilderClear(&debugString);
}

//------------------------------------------------------------------
void SerialDebugAddChar(uint8_t data){
	StringBuilderAddChar(&debugString, data);
}

//------------------------------------------------------------------
void SerialDebugAdd(uint8_t *text){
	StringBuilderAddString(&debugString, text);
}

//------------------------------------------------------------------
void SerialDebugAddVal(int16_t val){
	StringBuilderAddVal(&debugString, (int32_t)val);
}

//------------------------------------------------------------------
void SerialDebugAddValWLen(int16_t val, uint8_t len){
	StringBuilderAddValWlen(&debugString, (int32_t)val, len, ' ');
}

//------------------------------------------------------------------
void SerialDebugAddValWDot(int16_t val, uint8_t dotpos){
	StringBuilderAddValWdot(&debugString, (int32_t)val, dotpos);
}

//------------------------------------------------------------------
void SerialDebugAddHexVal(uint8_t val){
	StringBuilderAddHexVal(&debugString, val);
}

//------------------------------------------------------------------
void SerialDebugAddHex32Val(uint32_t val){
	StringBuilderAddHex32Val(&debugString, val);
}

//------------------------------------------------------------------
void SerialDebugAddParam(uint8_t *text, int16_t val){
	StringBuilderAddString(&debugString, text);
	StringBuilderAddVal(&debugString, (int32_t)val);
}

//------------------------------------------------------------------
void SerialDebugMessage(uint8_t *text){
	StringBuilderAddString(&debugString, "\r\n");
	StringBuilderAddString(&debugString, text);

}

//------------------------------------------------------------------
uint8_t SerialDebugProcess(void){

	if (StringBuilderTrackCheck(&debugString) > 0){
		//SerialSendStringBuffHolding(DEBUG_USART, &debugString);
		SerialSendIntEnable(DEBUG_USART);

		if (StringBuilderTrackHomeCheck(&debugString) > 0){
			return 1;
		}
	}
	SerialDebugClear();
	return 0;
}

//------------------------------------------------------------------
uint8_t SerialDebugInputCheck(void){
	return CommandManCheck(&debugCommandString);
}

//--------------------------------------------------------
void SerialDebugCommandDecode(void){
	CommandManDecode(&debugCommandString, &debugRecCommand);
}

//------------------------------------------------------------------
void SerialDebugInputClear(void){
	CommandManClear(&debugCommandString);
}

//------------------------------------------------------------------
uint8_t SerialDebugCommandCheck(void){
	if (SerialDebugInputCheck() > 0){
		SerialDebugCommandDecode();
		SerialDebugInputClear();
		return 1;
	}
	return 0;
}

//------------------------------------------------------------------
COMMAND_STRUCT_T *SerialDebugCommandGet(void){
	return &debugRecCommand;
}

