/*
 * stringBuilder.c
 *
 *  Created on: 25.08.2018
 *      Author: Procesorowiec
 */

#include "system8xx.h"
#include "stringBuilder.h"



//-----------------------------------------------------------------------------------
void StringBuilderStringInit(STRINGBUFF_T *pStringBuff, uint8_t *buff, uint16_t len){
	pStringBuff->base = buff;
	pStringBuff->pt = pStringBuff->base;
	pStringBuff->i = 0;
	pStringBuff->len = len;
	pStringBuff->track = 0;
}

//-----------------------------------------------------------------------------------
uint8_t StringBuilderClear(STRINGBUFF_T *pStringBuff){
	if ((pStringBuff->i > 0) && (pStringBuff->track >= pStringBuff->i)){
		pStringBuff->pt = pStringBuff->base;
		*pStringBuff->pt = 0;
		pStringBuff->i = 0;
		pStringBuff->track = 0;
		return 1;
	}
	return 0;
}

//-----------------------------------------------------------------------------------
void StringBuilderMasterClear(STRINGBUFF_T *pStringBuff){
	pStringBuff->pt = pStringBuff->base;
	*pStringBuff->pt = 0;
	pStringBuff->i = 0;
	pStringBuff->track = 0;
}

//-----------------------------------------------------------------------------------
uint8_t StringBuilderCheck(STRINGBUFF_T *pStringBuff){
	if (pStringBuff->i > 0){
		return 1;
	}
	return 0;
}

//-----------------------------------------------------------------------------------
int16_t StringBuilderActualLenGet(STRINGBUFF_T *pStringBuff){
	return pStringBuff->i;
}

//-----------------------------------------------------------------------------------
uint8_t StringBuilderTrackCheck(STRINGBUFF_T *pStringBuff){
	if (pStringBuff->track < pStringBuff->i ){
		return 1;
	}
	return 0;
}

//-----------------------------------------------------------------------------------
uint8_t StringBuilderTrackHomeCheck(STRINGBUFF_T *pStringBuff){
	if (pStringBuff->track == 0){
		return 1;
	}
	return 0;
}

//-----------------------------------------------------------------------------------
void StringBuilderTrackSet(STRINGBUFF_T *pStringBuff, int16_t track){
	pStringBuff->track = track;
}

//-----------------------------------------------------------------------------------
uint8_t *StringBuilderGet(STRINGBUFF_T *pStringBuff){
	return pStringBuff->base;
}

//-----------------------------------------------------------------------------------
uint8_t StringBuilderAddChar(STRINGBUFF_T *pStringBuff, uint8_t data){
	if (pStringBuff->i < (pStringBuff->len - 2)){
		pStringBuff->pt = (pStringBuff->base + pStringBuff->i);
		*pStringBuff->pt = data;
		pStringBuff->pt++;
		*pStringBuff->pt = 0;
		pStringBuff->i++;
		return 1;
	}
	return 0;
}

//-----------------------------------------------------------------------------------
void StringBuilderAddString(STRINGBUFF_T *pStringBuff, uint8_t *src){
	while ((src > 0) && (*src > 0)){
		if (StringBuilderAddChar(pStringBuff, *src) > 0){
			src++;
		} else {
			break;
		}
	}
}

//-----------------------------------------------------------------------------------
uint32_t StringBuilderAddVal(STRINGBUFF_T *pStringBuff, int32_t val){
	uint8_t buff[20];
	int32_t i = 0;
	uint32_t result = 0;

	if (val < 0){
		val *= -1;
		StringBuilderAddChar(pStringBuff, '-');
	}

	for (; i < 18; i++){
		buff[i] = (val % 10) + '0';
		val /= 10;
		result++;

		if (val == 0) break;
	}

	for (; i >= 0; i--){
		StringBuilderAddChar(pStringBuff, (buff[i]));
	}

	return result;
}

//-------------------------------------------------------------------
void StringBuilderAddValWlen(STRINGBUFF_T *pStringBuff, int32_t val, uint8_t len, uint8_t emptyChar){
	uint8_t buff[20];
	int32_t i = 0;
	uint32_t result = 0;

	if (val < 0){
		val *= -1;
		StringBuilderAddChar(pStringBuff, '-');
	}

	for (; i < 18; i++){
		buff[i] = (val % 10) + '0';
		val /= 10;
		result++;

		if (val == 0) break;
	}

	while (result < len){
		i++;
		buff[i] = emptyChar;
		result++;
	}
	if (result > len){
		i = (len - 1);
	}

	for (; i >= 0; i--){
		StringBuilderAddChar(pStringBuff, (buff[i]));
	}
}


//-------------------------------------------------------------------
void StringBuilderAddValWdot(STRINGBUFF_T *pStringBuff, int32_t val, uint16_t dotPos){
	uint8_t buff[20];
	int32_t i = 0;
	uint32_t result = 0;

	if (val < 0){
		val *= -1;
		StringBuilderAddChar(pStringBuff, '-');
	}

	for (; i < 16; i++){
		buff[i] = (val % 10) + '0';
		val /= 10;
		result++;

		if (val == 0) break;
	}

	while (result <= dotPos){
		i++;
		buff[i] = '0';
		result++;
	}

	for (; i >= 0; i--){
		StringBuilderAddChar(pStringBuff, (buff[i]));
		result--;

		if (result == dotPos){
			StringBuilderAddChar(pStringBuff, ',');
		}
	}
}

//-------------------------------------------------------------------
void StringBuilderAddHexVal(STRINGBUFF_T *pStringBuff, uint8_t val){
	uint8_t half;

	half = (val & 0xF0);
	half >>= 4;
	if (half <= 9) {
		StringBuilderAddChar(pStringBuff, (half + '0'));
	} else {
		StringBuilderAddChar(pStringBuff, ((half - 10) + 'A'));
	}

	half = (val & 0x0F);
	if (half <= 9) {
		StringBuilderAddChar(pStringBuff, (half + '0'));
	} else {
		StringBuilderAddChar(pStringBuff, ((half - 10) + 'A'));
	}
}

//-------------------------------------------------------------------
void StringBuilderAddHex32Val(STRINGBUFF_T *pStringBuff, int32_t val){
	StringBuilderAddChar(pStringBuff, 'x');
	StringBuilderAddHexVal(pStringBuff, ((val>>24) & 0xFF));
	StringBuilderAddHexVal(pStringBuff, ((val>>16) & 0xFF));
	StringBuilderAddHexVal(pStringBuff, ((val>>8) & 0xFF));
	StringBuilderAddHexVal(pStringBuff, (val & 0xFF));
}

