/*
 * system8xx.c
 *
 *  Created on: Feb 23, 2018
 *      Author: dsosnowski
 */


#include "chip.h"
#include "system8xx.h"


//--------------------------------------------------------------------------
void System8xxInit(void){

	/* Turn on the IRC by clearing the power down bit */
	Chip_SYSCTL_PowerUp(SYSCTL_SLPWAKE_IRC_PD);

	/* Select the PLL input in the IRC */
	Chip_Clock_SetSystemPLLSource(SYSCTL_PLLCLKSRC_IRC);

	/* Power down PLL to change the PLL divider ratio */
	Chip_SYSCTL_PowerDown(SYSCTL_SLPWAKE_SYSPLL_PD);

	//60MHz main clock (not system clock)(system clock == core clock)
	Chip_Clock_SetupSystemPLL(4, 1);

	/* Turn on the PLL by clearing the power down bit */
	Chip_SYSCTL_PowerUp(SYSCTL_SLPWAKE_SYSPLL_PD);

	/* Wait for PLL to lock */
	while (!Chip_Clock_IsSystemPLLLocked()) {}

	/* Setup FLASH access to 2 clocks (up to 30MHz) */
	Chip_FMC_SetFLASHAccess(FLASHTIM_30MHZ_CPU);

	/* Set system clock divider to 2 */
	//core clock = 60/2 = 30
	Chip_Clock_SetSysClockDiv(2);

	/* Set main clock source to the system PLL. This will drive 24MHz
	   for the main clock and 24MHz for the system clock */
	Chip_Clock_SetMainClockSource(SYSCTL_MAINCLKSRC_PLLOUT);

	SystemPeripInit();
}

//--------------------------------------------------------------------------
void SystemPeripInit(void){
	Chip_Clock_EnablePeriphClock(SYSCTL_CLOCK_SWM);
	//Chip_SWM_DisableFixedPin(SWM_FIXED_SWCLK);
	//Chip_SWM_DisableFixedPin(SWM_FIXED_SWDIO);

	Chip_Clock_EnablePeriphClock(SYSCTL_CLOCK_IOCON);
	Chip_Clock_EnablePeriphClock(SYSCTL_CLOCK_GPIO);
	Chip_Clock_EnablePeriphClock(SYSCTL_CLOCK_MRT);
	Chip_SYSCTL_PeriphReset(RESET_MRT);

	Chip_Clock_SetIOCONCLKDIV(IOCONCLKDIV0, 8);		//set clock for glitch filter
}

//--------------------------------------------------------------------------
void SystemInitEnd(void){
	Chip_Clock_DisablePeriphClock(SYSCTL_CLOCK_IOCON);
	Chip_Clock_EnablePeriphClock(SYSCTL_CLOCK_SWM);
}

//---------------------------------------------------
void IoMaskedValueSet(LPC_GPIO_T *pGPIO, uint8_t port, uint32_t mask, uint32_t val){
	Chip_GPIO_SetPortMask(pGPIO, port, ~(mask));
	Chip_GPIO_SetMaskedPortValue(pGPIO, port, val);
}

//--------------------------------------------------------------------------
void SystemSysTickInit(uint32_t intervalMs){
	SysTick->LOAD = (((MAIN_CLOCK / 1000) * intervalMs) - 1);
	SysTick->VAL = 0;
	SysTick->CTRL = (1<<0 | 1<<2);		//disable int
}

//--------------------------------------------------------------------------
uint8_t SystemSysTickCheck(void){
	if (SysTick->CTRL & (1<<16)){
		return 1;
	}
	return 0;
}


//--------------------------------------------------------------------------
const uint8_t ioconMap[] = {
		IOCON_PIO0,
		IOCON_PIO1,
		IOCON_PIO2,
		IOCON_PIO3,
		IOCON_PIO4,
		IOCON_PIO5,
		IOCON_PIO6,
		IOCON_PIO7,
		IOCON_PIO8,
		IOCON_PIO9,

		IOCON_PIO10,
		IOCON_PIO11,
		IOCON_PIO12,
		IOCON_PIO13,
		IOCON_PIO14,
		IOCON_PIO15,
		IOCON_PIO16,
		IOCON_PIO17,
		IOCON_PIO18,
		IOCON_PIO19,

		IOCON_PIO20,
		IOCON_PIO21,
		IOCON_PIO22,
		IOCON_PIO23,
		IOCON_PIO24,
		IOCON_PIO25,
		IOCON_PIO26,
		IOCON_PIO27,
		IOCON_PIO28,
};

//--------------------------------------------------------------------------
void IoConPinMuxSet(LPC_IOCON_T *pIOCON, uint8_t port, uint8_t pin, uint32_t modefunc){
	CHIP_PINx_T ioConPin = (ioconMap[(port*32) + pin]);
	Chip_IOCON_PinMuxSet(pIOCON, ioConPin, modefunc);
}

//--------------------------------------------------------------------------
void SystemPortInit(PINEX_STRUCT_T *portArray, int32_t len){
	for (int32_t i = 0; i < len; i++){
		Chip_GPIO_SetPinDIR(LPC_GPIO_PORT, portArray->port, portArray->pin, portArray->output);
		IoConPinMuxSet(LPC_IOCON, portArray->port, portArray->pin, portArray->iocon);
		Chip_GPIO_SetPinState(LPC_GPIO_PORT, portArray->port, portArray->pin, portArray->idle);
		portArray++;
	}
}
